(define (ast-type x)
  (define (builtin? x)
    (case x
      ('+ #t)
      ('- #t)
      ('* #t)
      ('! #t)
      ('= #t)
      ('bool->int #t)
      (else #f)))
  (cond
   ((list? x)
    (case (car x)
      ('if 'if)
      ('let 'let)
      ('lambda 'lambda)
      ('closure 'closure) ; only available in codegen
      (else 'app)))
   ((builtin? x) 'builtin)
   ((symbol? x) 'var)
   ((integer? x) 'int-literal)
   ((boolean? x) 'bool-literal)))

(define (ast-traverse f x)
  (case (ast-type x)
    ('let `(let ,(map (lambda (x) (list (car x) (f (cadr x))))
		      (let-bindings x))
	     ,@(map f (let-body e))))
    ('app (map f x))
    ('lambda `(lambda ,(lambda-args x) ,(f lambda-body)))
    ('if `(if ,@(map f (cdr x))))
    (else x)))

(define let-bindings cadr)
(define let-body cddr)

(define (lambda? x)
  (and (list? x) (eq? (car x) 'lambda)))

; for use in normalized form
(define lambda-arg caadr)
; for use elsewhere
(define lambda-args cadr)
(define lambda-body caddr)
