(load "codegen.scm")
(load "typecheck.scm")

(define (test actual expected)
  (when (not (equal? actual expected))
    (error #f
	   (format "test failed:\nexpected: ~a\nactual:   ~a"
		   expected actual))))

(define (read-file file)
  (call-with-input-file file
    (lambda (p)
      (let loop ((next (read-char p))
                 (result ""))
        (if (eof-object? next)
            result
            (loop (read-char p) (string-append result (string next))))))))

(define (test-prog prog exit-code)
  (compile-to-binary prog "/tmp/test-prog")
  (system "/tmp/test-prog"))

(define (test-prog-stdout prog output)
  (compile-to-binary prog "/tmp/test-prog")
  (system "/tmp/test-prog > /tmp/test-output.txt")
  (let ((str (read-file "/tmp/test-output.txt")))
    (test (substring str 0 (- (string-length str) 1))
	  output)))

(test (typecheck '(lambda (x) (+ ((lambda (y) (x y 3)) 5) 2)))
      '(abs (abs int (abs int int)) int))

(test-prog '(+ 1 2) 3)
(test-prog '((lambda (x) ((lambda (y) (+ x y)) 42)) 100) 142)
(test-prog '(let ((x (+ 1 32))
		  (y x))
	      ((lambda (z) (+ 1 z)) (* y x)))
	   1090)
(test-prog '(if ((lambda (x) (= x 2)) 1) 0 (- 32 1)) 31)

